(function () {
    hbHitchLog("ajax_hook", "start");

    //===========================Fetch=======================//

    if(window.fetch) {
        var _fetch = window.fetch;
        window.fetch = function (url, data) {
            var resp, reject;
            var p = new Promise(function(_success, _reject){
                resp = _success;
                reject = _reject
            });
            var fetchId = 'fetchId' + uuidv4();
            window.OMTFetch.nativePost({
                "fetchId": fetchId,
                "url": url,
                "method":data['method'],
                "headers":data['headers'],
                "body": data['body']
            });
            window.OMTFetch.hookedFetch[fetchId] = {
                "fetchId": fetchId,
                "url": url,
                "method":data['method'],
                "success": resp,
                "reject": reject
            };
            return p;
        };
    }

    window.OMTFetch = {
        hookedFetch: {},
        nativePost: nativePost,
        fetchNativeCallback: fetchNativeCallback
    };

    window.hb_hitch_fetch_callback = function(data) {
        var fetchId = data["fetchId"];
        var statusCode = data["statusCode"];
        var responseText = data["responseText"];
        var responseHeaders = data["responseHeaders"];
        var error = data["error"];
        window.OMTFetch.fetchNativeCallback(fetchId, statusCode, responseText, responseHeaders, error);
        hbHitchLog("ajax_callback", "hb_hitch_fetch_callback() window.OMTFetch.fetchNativeCallback()")
    };

    function fetchNativeCallback(fetchId, statusCode, responseText, responseHeaders, error){
        var fe = window.OMTFetch.hookedFetch[fetchId];
        if(fe === undefined || fe == null) {
            return;
        }
        if(error) {
            console.log(error);
            fe["reject"](error);
        } else {
            fe["success"](new Response(responseText));
            JSON.parse(responseText)
        }
    }

    function nativePost(params) {
        const classMap = "business/ajax";
        const method = "fetchRequestPreRequest";
        const data = {
            classMap: classMap,
            method: method,
            params: JSON.stringify(params)
        };
        //请求 Native 和Android有什么区别？
//        window.webkit.messageHandlers.hybirdBridge.postMessage(params)
        window.AppFunctions.callNative(JSON.stringify(data));
        hbHitchLog("ajax_callback", "fetch() window.AppFunctions.callNative data="+JSON.stringify(data));
    }


    //===========================AJAX=======================//

    window.OMTAjax = {
        hookedXHR: {},
        hookAjax: hookAjax,
        nativeAjaxPost: nativeAjaxPost,
        nativeCallback: nativeCallback,
    };

    window.hb_hitch_ajax_callback = function (data) {
        hbHitchLog("ajax_callback-string", JSON.stringify(data));
        var xhrId = data["xhrId"];
        var statusCode = data["statusCode"];
        var responseText = data["responseText"];
        var responseHeaders = data["responseHeaders"];
        var error = data["error"];
        window.OMTAjax.nativeCallback(xhrId, statusCode, responseText, responseHeaders, error);
    };

    function nativeAjaxPost(params) {
        const classMap = "business/ajax";
        const method = "ajaxRequestPreRequest";
        const data = {
            classMap: classMap,
            method: method,
            params: JSON.stringify(params)
        };
        //请求 Native 和Android有什么区别？
//        window.webkit.messageHandlers.hybirdBridge.postMessage(params)
        window.AppFunctions.callNative(JSON.stringify(data));
        hbHitchLog("ajax_callback", "fetch() window.AppFunctions.callNative data="+JSON.stringify(data));
    }

    function nativeCallback(xhrId, statusCode, responseText, responseHeaders, error) {
        hbHitchLog("ajax_callback", xhrId+"/"+statusCode+"/"+responseText+"/"+responseHeaders+"/"+error);
        var xhr = window.OMTAjax.hookedXHR[xhrId];
        if (xhr === undefined || xhr == null) {
            return;
        }
        if (xhr.isAborted) { // 如果该请求已经手动取消了
            return;
        }
        hbHitchLog("ajax_callback", "error:"+error);
        if (error) {
            xhr.readyState = 1;
            hbHitchLog("ajax_callback", "error:" + error + "statusCode:" + statusCode + "responseText:" + responseText + "responseHeaders:" + responseHeaders);
            if (xhr.onerror) {
                hbHitchLog("ajax_callback", "onerror");
                xhr.onerror();
            }
        } else {
            xhr.status = statusCode;
            xhr.responseText = responseText;
            xhr.readyState = 4;
            xhr.DONE = 4;
            if (xhr.responseType === 'json') {
                xhr.response = JSON.parse(responseText);
            } else {
                xhr.response = responseText
            }
            //xhr.response = responseText;
            xhr.omtResponseHeaders = responseHeaders;
            if (xhr.onreadystatechange) {
                xhr.onreadystatechange();
            }
            if (xhr.onload) {
                xhr.onload();
            }
            if (xhr.onloadend) {
                xhr.onloadend();
            }
        }
    }

    // hook ajax send 方法
    window.OMTAjax.hookAjax({
        setRequestHeader: function (arg, xhr) {
            hbHitchLog("ajax_callback", "setRequestHeader. arg[0]"+arg[0]);
            if (arg[0] === "__native_request_value__") {
                this.request_key = +arg[1];
                hbHitchLog("this.request_key", arg[0]);
            } else if (arg[0] === "__native_cache_key__") {
                this.cache_key = arg[1];
            } else {
                if (!this.omtHeaders) {
                    this.omtHeaders = {};
                }
                this.omtHeaders[arg[0]] = arg[1];
                hbHitchLog("ajax_callback", "this.omtHeaders. 0="+arg[0]+ " 1="+arg[1]);
            }
        },
        getAllResponseHeaders: function (arg, xhr) {
            var headers = this.omtResponseHeaders;
            if (headers) {
                if (typeof (headers) === 'object') {
                    var result = '';
                    for (var key in headers) {
                        result = result + key + ':' + headers[key] + '\r\n'
                    }
                    return result;
                }
                return headers;
            }
        },
        getResponseHeader: function (arg, xhr) {
            if (this.omtResponseHeaders && this.omtResponseHeaders(arg[0])) {
                return this.omtResponseHeaders(arg[0]);
            }
        },
        open: function (arg, xhr) {
            hbHitchLog("ajax_callback", "open", arg);
            this.omtOpenArg = arg;
        },
        send: function (arg, xhr) {
            // 通过 return true 可以阻止默认 Ajax 请求，不返回则会继续原来的请求
            debugger;
            this.isAborted = false;
            var url = this.omtOpenArg[1]
            var xhrId = 'xhrId' + uuidv4();
            window.OMTAjax.hookedXHR[xhrId] = this;

            var method = this.omtOpenArg[0];
            if (method == 'POST') {
                var data = arg[0]
                if (data) {
                    window.OMTAjax.nativeAjaxPost({
                        "xhrId": xhrId,
                        "url": url,
                        "method":'POST',
                        'header':this.omtHeaders,
                        "body": arg[0]
                    })
                    return true;
                }
            }
            return false;
        },
        abort: function (arg, xhr) {
            hbHitchLog("ajax_callback", "abort");
            if (this.omtOpenArg[0] === 'POST') {
                if (xhr.onabort) {
                    xhr.onabort()
                }
                return true;
            }
        }
    });

    function hookAjax(proxy) {
        // 保存真正的XMLHttpRequest对象
        window._ahrealxhr = window._ahrealxhr || XMLHttpRequest;

        XMLHttpRequest = function () {
            var xhr = new window._ahrealxhr;
            // 直接在一个对象上定义一个新属性，或者修改一个对象的现有属性， 并返回这个对象
            Object.defineProperty(this, 'xhr', {
                value: xhr
            })
        };

        // 获取 XMLHttpRequest 对象的属性
        var prototype = window._ahrealxhr.prototype;

        for (var attr in prototype) {
            var type = "";

            try {
                type = typeof prototype[attr]
            } catch (e) {
            }

            if (type === "function") {
                XMLHttpRequest.prototype[attr] = hookfunc(attr);
            } else {
                // 给属性提供 getter、setter 方法
                Object.defineProperty(XMLHttpRequest.prototype, attr, {
                    get: getFactory(attr),
                    set: setFactory(attr),
                    enumerable: true
                })
            }
        }

        function getFactory(attr) {
            return function () {
                // 判断对象是否包含特定的自身（非继承）属性
                var v = this.hasOwnProperty(attr + "_") ? this[attr + "_"] : this.xhr[attr];
                var attrGetterHook = (proxy[attr] || {})["getter"];
                return attrGetterHook && attrGetterHook(v, this) || v
            }
        }

        function setFactory(attr) {
            return function (v) {
                var xhr = this.xhr;
                var that = this;
                var hook = proxy[attr];

                // 回调属性 onreadystatechange 等
                if (typeof hook === "function") {
                    xhr[attr] = function () {

                        hook.call(that, xhr) || v.apply(xhr, arguments);
                    }
                } else {
                    //If the attribute isn't writeable, generate proxy attribute
                    var attrSetterHook = (hook || {})["setter"];
                    v = attrSetterHook && attrSetterHook(v, that) || v;

                    xhr[attr] = v;
                    this[attr + "_"] = v;
                }
            }
        }

        function hookfunc(func) {
            return function () {
                var args = [].slice.call(arguments);
                // call() 方法调用一个函数, 其具有一个指定的this值和分别地提供的参数
                // 该方法的作用和 apply() 方法类似，只有一个区别，就是call()方法接受的是若干个参数的列表，而apply()方法接受的是一个包含多个参数的数组
                if (proxy[func]) {
                    var result = proxy[func].call(this, args, this.xhr);
                    if (result) {
                        return result;
                    }
                }

                return this.xhr[func].apply(this.xhr, args);
            }
        }

        return window._ahrealxhr;
    }

    function uuidv4() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0, v = c === 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }

    function unHookAjax() {
        if (window._ahrealxhr)
            XMLHttpRequest = window._ahrealxhr;
        window._ahrealxhr = undefined;
    }

    function hbHitchLog(tag, msg){
        console.log(tag + ": " + msg)
    }

    hbHitchLog("ajax_hook", "end");
})();